/*
**  GEcho 1.02+ Developers Kit Demonstration Source Code
**
**  Written by Gerard J. van der Land
**
**  Copyright (C) 1992-1994 Gerard J. van der Land. All rights reserved.
**
**  Last updates: 09-Feb-94
*/

#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>

typedef unsigned char  byte;
typedef unsigned int   word;
typedef unsigned long  dword;

#include "gestruct.h"

SETUP_GE     sys;
AREAFILE_HDR AreaHdr;
AREAFILE_GE  Area;
AREAFILE_GEX *AreaB, *AreaP;
CONNECTION   Connection[MAXCONNECTIONS], *ConnectionP;

int          SetupGE, AreaFileGE, AreaFileGEX;
word         areas, arearecsize, connectionsize;

int main(void)
{
   register word counter, result, records, node;
   long temp;

/*
**  Opening and reading SETUP.GE
*/
   SetupGE = open("SETUP.GE", O_RDONLY|O_DENYWRITE|O_BINARY);
   if (SetupGE == -1)
   {
      printf("Unable to open SETUP.GE\n");
      return(255);
   }
   result = read(SetupGE, &sys, sizeof(SETUP_GE));
   close(SetupGE);

   if (result < sizeof(SETUP_GE))
   {
      printf("Error reading SETUP.GE\n");
      return(255);
   }

   if (sys.sysrev != GE_THISREV)
   {
      printf("System file revision level mismatch\n");
      return(251);
   }

   printf("System file created by GEcho %u.%02u\n", sys.version_major, sys.version_minor);
   printf("SysOp name: %s\n", sys.username[0]);

/*
**  Opening AREAFILE.GE and checking the header
*/
   AreaFileGE = open("AREAFILE.GE", O_RDWR|O_DENYWRITE|O_BINARY);
   if (AreaFileGE == -1)
   {
      printf("Unable to open AREAFILE.GE\n");
      return(255);
   }

   result = read(AreaFileGE, &AreaHdr, sizeof(AREAFILE_HDR));
   if (result < sizeof(AREAFILE_HDR))
   {
      printf("Error reading AREAFILE header\n");
      return(255);
   }

   if (AreaHdr.hdrsize < sizeof(AREAFILE_HDR) ||
       AreaHdr.recsize < sizeof(AREAFILE_GE))
   {
      printf("Incompatible AREAFILE record size\n");
      return(251);
   }

   connectionsize = AreaHdr.maxconnections * sizeof(CONNECTION);
   arearecsize = AreaHdr.recsize + connectionsize;

/*
**  Reading AREAFILE.GE using AREAFILE.GEX index file
*/

   AreaFileGEX = open("AREAFILE.GEX", O_RDWR|O_DENYWRITE|O_BINARY);
   if (AreaFileGEX == -1)
   {
      printf("Unable to open AREAFILE.GEX\n");
      return(255);
   }

   temp = filelength(AreaFileGEX) / sizeof(AREAFILE_GEX);
   if (temp > MAXAREAS)
   {
      printf("Index file too large\n");
      close(AreaFileGEX);
      return(255);
   }
   areas = (word) temp;

   AreaB = malloc(areas * sizeof(AREAFILE_GEX));
   if (AreaB == NULL)
   {
      printf("Insufficient memory\n");
      close(AreaFileGEX);
      return(254);
   }

   read(AreaFileGEX, AreaB, areas * sizeof(AREAFILE_GEX));
   close(AreaFileGEX);

   for (AreaP = AreaB, counter = 1; counter <= areas; counter++, AreaP++)
   {
      lseek(AreaFileGE, AreaP->offset, SEEK_SET);
      result = read(AreaFileGE, &Area, sizeof(AREAFILE_GE));
      if (result == sizeof(AREAFILE_GE))
      {
	 printf("%4u  %s\n", counter, Area.name);

	 /* Print list of nodes connected to this area */
	 lseek(AreaFileGE, AreaP->offset + AreaHdr.recsize, SEEK_SET);
	 read(AreaFileGE, Connection, connectionsize);

	 ConnectionP = Connection;
	 for (node = 1; node <= AreaHdr.maxconnections && ConnectionP->address.zone; node++, ConnectionP++)
	 {
	    printf("      %3u  %u:%u/%u.%u\n", node,
					       ConnectionP->address.zone,
					       ConnectionP->address.net,
					       ConnectionP->address.node,
					       ConnectionP->address.point);
	 }
      }
   }
   free(AreaB);

/*
**  Reading AREAFILE.GE sequentially
*/

   records = (filelength(AreaFileGE) - AreaHdr.hdrsize) / arearecsize;

   for (areas = 0, counter = 0; counter < records; counter++)
   {
      lseek(AreaFileGE, (long) AreaHdr.hdrsize + (long) arearecsize * counter, SEEK_SET);
      result = read(AreaFileGE, &Area, sizeof(AREAFILE_GE));
      if (result < sizeof(AREAFILE_GE)) break;
      if ((Area.options & REMOVED) == 0)
      {
	 areas++;
	 printf("%4u  %s\n", areas, Area.name);
      }
   }
   close(AreaFileGE);
   return(0);
}

/* end of file "ge_demo.c" */
